@@ -12,18 +12,13 @@ except ImportError: import parsedatetime.parsedatetime as pdt
 import re
 from datetime import datetime
 import time
-try: import simplejson as json
-except ImportError: import json
 import sys
-import glob
 try:
     from Crypto.Cipher import AES
     from Crypto import Random
     crypto_installed = True
 except ImportError:
     crypto_installed = False
-if "win32" in sys.platform: import pyreadline as readline
-else: import readline
 import hashlib
 try:
     import colorama
@@ -34,13 +29,11 @@ import plistlib
 import pytz
 import uuid
 
-
 class Journal(object):
-    def __init__(self, **kwargs):
+    def __init__(self, name='default', **kwargs):
         self.config = {
             'journal': "journal.txt",
             'encrypt': False,
-            'password': "",
             'default_hour': 9,
             'default_minute': 0,
             'timeformat': "%Y-%m-%d %H:%M",
@@ -54,7 +47,8 @@ class Journal(object):
         consts.DOWParseStyle = -1  # "Monday" will be either today or the last Monday
         self.dateparse = pdt.Calendar(consts)
         self.key = None  # used to decrypt and encrypt the journal
-        self.search_tags = None # Store tags we're highlighting
+        self.search_tags = None  # Store tags we're highlighting
+        self.name = name
 
         journal_txt = self.open()
         self.entries = self.parse(journal_txt)
@@ -77,7 +71,7 @@ class Journal(object):
             plain = crypto.decrypt(cipher[16:])
         except ValueError:
             util.prompt("ERROR: Your journal file seems to be corrupted. You do have a backup, don't you?")
-            sys.exit(-1)
+            sys.exit(1)
         padding = " ".encode("utf-8")
         if not plain.endswith(padding):  # Journals are always padded
             return None
@@ -95,33 +89,29 @@ class Journal(object):
         plain += b" " * (AES.block_size - len(plain) % AES.block_size)
         return iv + crypto.encrypt(plain)
 
-    def make_key(self, prompt="Password: "):
+    def make_key(self, password):
         """Creates an encryption key from the default password or prompts for a new password."""
-        password = self.config['password'] or util.getpass(prompt)
         self.key = hashlib.sha256(password.encode("utf-8")).digest()
 
     def open(self, filename=None):
         """Opens the journal file defined in the config and parses it into a list of Entries.
         Entries have the form (date, title, body)."""
         filename = filename or self.config['journal']
-        journal = None
+
         if self.config['encrypt']:
             with open(filename, "rb") as f:
-                journal = f.read()
-            decrypted = None
-            attempts = 0
-            while decrypted is None:
-                self.make_key()
-                decrypted = self._decrypt(journal)
-                if decrypted is None:
-                    attempts += 1
-                    self.config['password'] = None  # This password doesn't work.
-                    if attempts < 3:
-                        util.prompt("Wrong password, try again.")
-                    else:
-                        util.prompt("Extremely wrong password.")
-                        sys.exit(-1)
-            journal = decrypted
+                journal_encrypted = f.read()
+
+            def validate_password(password):
+                self.make_key(password)
+                return self._decrypt(journal_encrypted)
+
+            # Soft-deprecated:
+            journal = None
+            if 'password' in self.config:
+                journal = validate_password(self.config['password'])
+            if not journal:
+                journal = util.get_password(keychain=self.name, validator=validate_password)
         else:
             with codecs.open(filename, "r", "utf-8") as f:
                 journal = f.read()
@@ -141,12 +131,20 @@ class Journal(object):
         for line in journal.splitlines():
             try:
                 # try to parse line as date => new entry begins
+                line = line.strip()
                 new_date = datetime.strptime(line[:date_length], self.config['timeformat'])
 
                 # parsing successfull => save old entry and create new one
                 if new_date and current_entry:
                     entries.append(current_entry)
-                current_entry = Entry.Entry(self, date=new_date, title=line[date_length+1:])
+
+                if line.endswith("*"):
+                    starred = True
+                    line = line[:-1]
+                else:
+                    starred = False
+
+                current_entry = Entry.Entry(self, date=new_date, title=line[date_length+1:], starred=starred)
             except ValueError:
                 # Happens when we can't parse the start of the line as an date.
                 # In this case, just append line to our body.
@@ -161,9 +159,12 @@ class Journal(object):
         return entries
 
     def __unicode__(self):
+        return self.pprint()
+
+    def pprint(self, short=False):
         """Prettyprints the journal's entries"""
         sep = "\n"
-        pp = sep.join([e.pprint() for e in self.entries])
+        pp = sep.join([e.pprint(short=short) for e in self.entries])
         if self.config['highlight']:  # highlight tags
             if self.search_tags:
                 for tag in self.search_tags:
@@ -177,9 +178,6 @@ class Journal(object):
                             pp)
         return pp
 
-    def pprint(self):
-        return self.__unicode__()
-
     def __repr__(self):
         return "<Journal with {0} entries>".format(len(self.entries))
 
@@ -204,7 +202,7 @@ class Journal(object):
         if n:
             self.entries = self.entries[-n:]
 
-    def filter(self, tags=[], start_date=None, end_date=None, strict=False, short=False):
+    def filter(self, tags=[], start_date=None, end_date=None, starred=False, strict=False, short=False):
         """Removes all entries from the journal that don't match the filter.
 
         tags is a list of tags, each being a string that starts with one of the
@@ -212,6 +210,8 @@ class Journal(object):
 
         start_date and end_date define a timespan by which to filter.
 
+        starred limits journal to starred entries
+
         If strict is True, all tags must be present in an entry. If false, the
         entry is kept if any tag is present."""
         self.search_tags = set([tag.lower() for tag in tags])
@@ -222,6 +222,7 @@ class Journal(object):
         result = [
             entry for entry in self.entries
             if (not tags or tagged(entry.tags))
+            and (not starred or entry.starred)
             and (not start_date or entry.date > start_date)
             and (not end_date or entry.date < end_date)
         ]
@@ -241,17 +242,23 @@ class Journal(object):
                     e.body = ''
         self.entries = result
 
-    def parse_date(self, date):
+    def parse_date(self, date_str):
         """Parses a string containing a fuzzy date and returns a datetime.datetime object"""
-        if not date:
+        if not date_str:
             return None
-        elif type(date) is datetime:
-            return date
+        elif isinstance(date_str, datetime):
+            return date_str
 
-        date, flag = self.dateparse.parse(date)
+        date, flag = self.dateparse.parse(date_str)
 
         if not flag:  # Oops, unparsable.
-            return None
+            try:  # Try and parse this as a single year
+                year = int(date_str)
+                return datetime(year, 1, 1)
+            except ValueError:
+                return None
+            except TypeError:
+                return None
 
         if flag is 1:  # Date found, but no time. Use the default time.
             date = datetime(*date[:3], hour=self.config['default_hour'], minute=self.config['default_minute'])
@@ -272,7 +279,7 @@ class Journal(object):
         If a date is given, it will parse and use this, otherwise scan for a date in the input first."""
 
         raw = raw.replace('\\n ', '\n').replace('\\n', '\n')
-
+        starred = False
         # Split raw text into title and body
         title_end = len(raw)
         for separator in ["\n", ". ", "? ", "! "]:
@@ -281,15 +288,22 @@ class Journal(object):
                 title_end = sep_pos
         title = raw[:title_end+1]
         body = raw[title_end+1:].strip()
+        starred = False
         if not date:
             if title.find(":") > 0:
+                starred = "*" in title[:title.find(":")]
                 date = self.parse_date(title[:title.find(":")])
-                if date:  # Parsed successfully, strip that from the raw text
+                if date or starred:  # Parsed successfully, strip that from the raw text
                     title = title[title.find(":")+1:].strip()
+            elif title.strip().startswith("*"):
+                starred = True
+                title = title[1:].strip()
+            elif title.strip().endswith("*"):
+                starred = True
+                title = title[:-1].strip()
         if not date:  # Still nothing? Meh, just live in the moment.
             date = self.parse_date("now")
-
-        entry = Entry.Entry(self, date, title, body)
+        entry = Entry.Entry(self, date, title, body, starred=starred)
         self.entries.append(entry)
         if sort:
             self.sort()
